using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;

/// <summary>
///     The FitConnect API Client
/// </summary>
public class Client {
    private readonly string _clientId;
    private readonly string _clientSecret;
    private Sender _sender;
    private Subscriber _subscriber;
    private FitConnectApiService _service;


    public Client(FitConnectEndpoints endpoints,
        string clientId, string clientSecret,
        X509Certificate2? certificate = null,
        ILogger? logger = null) {
        _clientId = clientId;
        _clientSecret = clientSecret;
        _sender = new Sender(logger, endpoints, certificate);
        _subscriber = new Subscriber(logger, endpoints, certificate);
        _service = new FitConnectApiService(endpoints, logger);
    }

    /// <summary>
    ///     Sending a submission to the FitConnect API<br/>
    /// <para>The submission is prepared for sending to the FitConnect API.</para>
    /// </summary>
    /// <param name="submission">The submission to send</param>
    /// <returns>success</returns>
    public async Task<bool> SendSubmissionAsync(Submission submission) {
        // Einreichung anlegen und ankündigen
        var createSubmissionDto = await _sender.CreateSubmissionDtoAsync(submission);

        // Anlagen verschlüsseln
        var encryptedAttachments = submission.Attachments
            .Select(a => _sender.EncryptAttachment(a)).ToList();

        // Anlagen hochladen
        await _sender.UploadAttachmentsAsync(encryptedAttachments);

        // Metadaten befüllen und verschlüsseln
        var metaData = await _sender.CreateMetadataAsync("", new byte[] { });

        // Fachdaten verschlüsseln
        var encryptedData = _sender.EncryptDataAsync(submission.Data);

        // Fachdaten & Metadaten hochladen und Einreichung absenden
        await _sender.SubmitSubmissionAsync(submission);

        return true;
    }

    /// <summary>
    ///     Gets all submissions from the FitConnect API
    /// </summary>
    /// <returns>List of the available submissions</returns>
    public async Task<List<Submission>> GetSubmissionsAsync(string destinationId, int offset = 0,
        int limit = 100) {
        var pickUpList =
            await _service.SubmissionService.ListSubmissions(destinationId, offset, limit);
        return pickUpList.Submissions.Select(dto => (Submission)dto).ToList();
    }

    /// <summary>
    ///     Sending a submission to the FitConnect API
    /// </summary>
    /// <param name="submission">The submission to send</param>
    /// <returns>success</returns>
    public bool SendSubmission(Submission submission) {
        return SendSubmissionAsync(submission).Result;
    }

    /// <summary>
    ///     Gets all submissions from the FitConnect API
    /// </summary>
    /// <returns>List of the available submissions</returns>
    public List<Submission> GetSubmissions(string destinationId, int offset = 0,
        int limit = 100) {
        return GetSubmissionsAsync(destinationId, offset, limit).Result;
    }
}
